/*
 *  PKUtilities.h
 *  PRODUCTION KusoGA Utility Library
 *
 *  Created by Decomo@Freyja on  07/01/06.
 *
 */

#ifndef _PKUTILITIES_H_
#define _PKUTILITIES_H_

// エンディアン定義
#if defined(__BIG_ENDIAN__)
#define _PK_BIG_ENDIAN_
#elif defined(__LITTLE_ENDIAN__)
#define _PK_LITTLE_ENDIAN_
#endif

// バイトスワップ関連
#define BS_BYTE1(x) (   (x)      & 0xFF )
#define BS_BYTE2(x) ( ( (x)>> 8 )& 0xFF )
#define BS_BYTE3(x) ( ( (x)>>16 )& 0xFF )
#define BS_BYTE4(x) ( ( (x)>>24 )& 0xFF )

#define BYTESWAP_INT16(x) ( (unsigned short)( (BS_BYTE1(x)<<8) | BS_BYTE2(x) ) )
#define BYTESWAP_INT32(x) ( BS_BYTE1(x)<<24 | BS_BYTE2(x)<<16 | BS_BYTE3(x)<<8 | BS_BYTE4(x) )

#ifdef _PK_BIG_ENDIAN_
#define INT16_TO_LE(x) ( BYTESWAP_INT16(x) )
#define INT32_TO_LE(x) ( BYTESWAP_INT32(x) )
#define INT16_TO_BE(x) (x)
#define INT32_TO_BE(x) (x)
#endif

#ifdef _PK_LITTLE_ENDIAN_
#define INT16_TO_LE(x) (x)
#define INT32_TO_LE(x) (x)
#define INT16_TO_BE(x) ( BYTESWAP_INT16(x) )
#define INT32_TO_BE(x) ( BYTESWAP_INT32(x) )
#endif

// 2/4バイトのchar配列型ビッグ/リトルエンディアン値をshort/int値に変換する
#define BE_ARRAY_TO_INT16(x) (unsigned short)( INT16_TO_BE( *(unsigned short*)x ) )
#define BE_ARRAY_TO_INT32(x) (unsigned int)( INT32_TO_BE( *(unsigned int*)x )
#define LE_ARRAY_TO_INT16(x) (unsigned short)( INT16_TO_LE( *(unsigned short*)x ) )
#define LE_ARRAY_TO_INT32(x) (unsigned int)( INT32_TO_LE( *(unsigned int*)x ) )

#endif // _PKUTILITIES_H_